@@ -1,16 +1,18 @@
 ;;; selected-window-contrast.el --- Highlight by brightness of text and background   -*- lexical-binding: t -*-
 
 ;; Copyright (c) 2025 github.com/Anoncheg1,codeberg.org/Anoncheg
-;; Author: <github.com/Anoncheg1,codeberg.org/Anoncheg>
+;; Author: Anoncheg <vitalij@gmx.com>
+;;
 ;; Keywords:  color, windows, faces, buffer, background
 ;; URL: https://codeberg.org/Anoncheg/selected-window-contrast
-;; Package-Version: 20260212.1150
-;; Package-Revision: 7936b00a5822
+;; Package-Version: 20260216.2005
+;; Package-Revision: b5a15be393b6
 ;; Created: 11 dec 2024
-;; Package-Requires: ((emacs "29.4"))
+;; Package-Requires: ((emacs "25.1"))
 ;; SPDX-License-Identifier: AGPL-3.0-or-later
 
 ;;; License
+
 ;; This file is not part of GNU Emacs.
 
 ;; This program is free software: you can redistribute it and/or modify
@@ -28,14 +30,17 @@
 ;; see <https://www.gnu.org/licenses/agpl-3.0.en.html>.
 
 ;;; Commentary:
+
 ;; Highlight selected window by adjusting contrast of text
 ;;  "foreground" and background.
-;; Working good if you switch temes frequently, contrast will be kept.
+;; Working good if you switch themes frequently, contrast will be kept.
 ;; Also this works for modeline.
-;;  We also highligh cursor position, this may be disabled with
-;;  M-x customize-variable RET selected-window-contrast-text-switch-mode
+;; We also highligh cursor position, this may be disabled with
+;;  (setopt selected-window-contrast-region-flag nil) in .emacs
+;;  or
+;;  M-x customize-variable RET selected-window-contrast-region-flag
 
-;; Usage:
+;;; Usage:
 
 ;; (add-to-list 'load-path "path_to/selected-window-contrast") ; optional
 ;; (when (require 'selected-window-contrast nil 'noerror)
@@ -46,12 +51,8 @@
 ;;   (add-hook 'buffer-list-update-hook
 ;;             #'selected-window-contrast-highlight-selected-window))
 
+;;;  How this works:
 
-;; To disable highlighting window with rectangle around pointer use:
-;; (setopt selected-window-contrast-mode 1)
-
-
-;; How this works:
 ;;  1) We get color with `face-attribute' `selected-frame' for
 ;;  foreground and backgraound.
 ;;  2) Convert color to HSL
@@ -61,12 +62,14 @@
 
 ;; Customize: M-x customize-group RET selected-window-contrast
 
-;; Donate:
+;;; Donate:
+
 ;; - BTC (Bitcoin) address: 1CcDWSQ2vgqv5LxZuWaHGW52B9fkT5io25
 ;; - USDT (Tether TRX-TRON) address: TVoXfYMkVYLnQZV3mGZ6GvmumuBfGsZzsN
 ;; - TON (Telegram) address: UQC8rjJFCHQkfdp7KmCkTZCb5dGzLFYe2TzsiZpfsnyTFt9D
 
-;; Other packages:
+;;; Other packages:
+
 ;; - Modern navigation in major modes https://github.com/Anoncheg1/firstly-search
 ;; - Search with Chinese	https://github.com/Anoncheg1/pinyin-isearch
 ;; - Ediff no 3-th window	https://github.com/Anoncheg1/ediffnw
@@ -78,77 +81,74 @@
 ;; - Call LLMs & AIfrom Org-mode block.  https://github.com/Anoncheg1/emacs-oai
 
 ;;; Code:
+
 ;; Touch: Global variables bound deep is not good, it is a type of the inversion of control.
 ;; I am the best that is why I am the winner.
 (require 'color)
 (require 'rect)
 
+(defgroup selected-window-contrast nil
+ "Highlight by brightness of text and background."
+ :group 'faces)
+
 ;; - configurable:
 (defcustom selected-window-contrast-bg-selected nil
   "Non-nil used to set selected window background contrast in [0-1] range.
 Higher value increase contrast between text and background.
 This value change contrast of text regarding to background."
-  :group 'selected-window-contrast
-  :type '(choice (number :tag "contrast in [0-1] range")
+  :type '(choice (restricted-sexp :match-alternatives
+                                  ((lambda (x) (and (numberp x) (<= 0 x 1))))
+                                  :message "Contrast value must be between 0 and 1")
                  (const :tag "Don't change default contrast of theme." nil)))
 
 (defcustom selected-window-contrast-bg-others 0.8
   "Non-nil used to set not selected windows background contrast.
 in [0-1] range."
-  :group 'selected-window-contrast
-  :type '(choice (number :tag "contrast [0-1].")
-                 (const :tag "Don't change default contrast of theme." nil)))
+  :type '(choice
+          (const :tag "Don't change default contrast of theme." nil)
+          (restricted-sexp :match-alternatives
+                           ((lambda (x) (and (numberp x) (<= 0 x 1))))
+                           :message "Contrast value must be between 0 and 1")))
 
 (defcustom selected-window-contrast-text-selected nil
   "Non-nil used to set not selected windows text contrast in [0-1] range."
-  :group 'selected-window-contrast
-  :type '(choice (number :tag "Text contrast [0-1].")
+  :type '(choice (restricted-sexp :match-alternatives
+                                  ((lambda (x) (and (numberp x) (<= 0 x 1))))
+                                  :message "Contrast value must be between 0 and 1")
                  (const :tag "Don't change default contrast of theme." nil)))
 
 (defcustom selected-window-contrast-text-others nil
   "Non-nil used to set not selected windows text contrast in [0-1] range."
-  :group 'selected-window-contrast
-  :type '(choice (number :tag "Text contrast [0-1].")
+  :type '(choice (restricted-sexp :match-alternatives
+                           ((lambda (x) (and (numberp x) (<= 0 x 1))))
+                           :message "Contrast value must be between 0 and 1")
                  (const :tag "Don't change default contrast of theme." nil)))
 
-(defcustom selected-window-contrast-mode 3
-  "Highlight window by 3 contrast and region, 1 by contrast, 2 by region."
-  :group 'selected-window-contrast
-  :type 'number)
+(defcustom selected-window-contrast-region-flag t
+  "Non-nil means enable highlighting cursor by region around it."
+  :type 'boolean)
+
+(defcustom selected-window-contrast-flag t
+  "Non-nil means enable highlighting by contrast."
+  :type 'boolean)
 
 (defcustom selected-window-contrast-region-timeout 0.5
-  "Hightlight cursor position: Second for which to show rectangle around."
-  :group 'selected-window-contrast
+  "Highlighting curson, second for which to show rectangle around."
   :type 'float)
 
+(defcustom selected-window-contrast-region-width 8
+  "Highlighting curson, width in chars of rectangle."
+  :type 'number)
+
+(defcustom selected-window-contrast-region-lines 2
+  "Highlighting curson, height in lines of rectangle."
+  :type 'number)
+
 (defun selected-window-contrast--get-current-colors ()
   "Get current text and background color of default face.
 Returns list: (foreground background), both strings."
-  (list (face-attribute 'default :foreground (selected-frame))
-        (face-attribute 'default :background (selected-frame))))
-
-(defun selected-window-contrast--parse-color (color)
-  "Return normalized RGB list for COLOR (hex or name).
-COLOR: string, hex like '#rrggbb' or '#rrrrggggbbbb', or color name."
-  (cond
-   ((and (stringp color) (string-prefix-p "#" color))
-    (let* ((hex (substring color 1))
-           (len (length hex))
-           (digits (/ len 3))
-           (maxval (if (= digits 2) 255.0 65535.0)))
-      (mapcar
-       (lambda (i)
-         (/ (string-to-number (substring hex (* i digits) (* (+ i 1) digits)) 16)
-            maxval))
-       '(0 1 2))))
-   ((and (stringp color) (color-name-to-rgb color))
-    (color-name-to-rgb color))
-   (t (error "Invalid color: %s" color))))
-
-(defun selected-window-contrast--color-to-hsl (color)
-  "Convert COLOR (hex or name) to a normalized HSL list.
-COLOR: string, hex or color name."
-  (apply #'color-rgb-to-hsl (selected-window-contrast--parse-color color)))
+  (list (face-attribute 'default :foreground)
+        (face-attribute 'default :background)))
 
 (defun selected-window-contrast-adjust-contrast (text-color background-color bg-mag text-mag)
   "Maximize visual contrast between TEXT-COLOR and BACKGROUND-COLOR.
@@ -162,27 +162,26 @@ Arguments:
  TEXT-MAG (float in [0,1], optional): stretching of contrast for text.
 Returns:
  List: (NEW-TEXT-RGB NEW-BACKGROUND-RGB), each as (R G B) floats in [0,1]."
-  (let ((text-hsl (selected-window-contrast--color-to-hsl text-color))
-        (bg-hsl   (selected-window-contrast--color-to-hsl background-color))
-        (mid 0.5))
-    (let ((res-text-rgb
-           (if (not text-mag)
-               (apply #'color-hsl-to-rgb text-hsl)
-             ;; else
-             (let* ((t-l (nth 2 text-hsl))
-                    (new-t-l (if (> t-l mid)
-                                 (+ mid (* text-mag (- 1.0 mid)))
-                               (- mid (* text-mag (- mid 0))))))
-               (apply #'color-hsl-to-rgb (list (nth 0 text-hsl) (nth 1 text-hsl) new-t-l)))))
-          (res-bg-rgb (if (not bg-mag)
-                          (apply #'color-hsl-to-rgb bg-hsl)
-                        ;; else
-                        (let* ((b-l (nth 2 bg-hsl))
-                               (new-b-l (if (> b-l mid)
-                                            (+ mid (* bg-mag (- 1.0 mid)))
-                                          (- mid (* bg-mag (- mid 0))))))
-                          (apply #'color-hsl-to-rgb (list (nth 0 bg-hsl) (nth 1 bg-hsl) new-b-l))))))
-      (list res-text-rgb res-bg-rgb))))
+  (let* ((text-hsl (apply #'color-rgb-to-hsl (color-name-to-rgb text-color)))
+         (bg-hsl   (apply #'color-rgb-to-hsl (color-name-to-rgb background-color)))
+         (mid 0.5))
+    (list
+     (if (not text-mag)
+         (apply #'color-hsl-to-rgb text-hsl)
+       ;; else
+       (let* ((t-l (nth 2 text-hsl))
+              (new-t-l (if (> t-l mid)
+                           (+ mid (* text-mag (- 1.0 mid)))
+                         (- mid (* text-mag (- mid 0))))))
+         (apply #'color-hsl-to-rgb (list (nth 0 text-hsl) (nth 1 text-hsl) new-t-l))))
+     (if (not bg-mag)
+         (apply #'color-hsl-to-rgb bg-hsl)
+       ;; else
+       (let* ((b-l (nth 2 bg-hsl))
+              (new-b-l (if (> b-l mid)
+                           (+ mid (* bg-mag (- 1.0 mid)))
+                         (- mid (* bg-mag (- mid 0))))))
+         (apply #'color-hsl-to-rgb (list (nth 0 bg-hsl) (nth 1 bg-hsl) new-b-l)))))))
 
 (defun selected-window-contrast--rgb-to-hex (rgb &optional digits)
   "Convert normalized RGB list to hex string.
@@ -248,19 +247,23 @@ or decrease contrast."
                             :background new-back)
         t))))
 
-(defvar selected-window-contrast-prev-window nil)
+(defvar selected-window-contrast-prev-window nil
+  "Saved current window, because `previous-window' is not working.
+Used in `selected-window-contrast-mark-small-rectangle-temporary'.")
 
 (defun selected-window-contrast-mark-small-rectangle-temporary (window)
   "Mark a 2x2 rectangle around point for 1 sec, to hightlight WINDOW.
 Use `rectangle-mark-mode'.  Deactivate rectangle after 1 second or less."
   (interactive)
   (when (and (eq window (selected-window))
-             (not (window-minibuffer-p window)))
+             (not (window-minibuffer-p window))
+             (not (window-minibuffer-p selected-window-contrast-prev-window)))
+
     ;; Enable rectangle selection.
-    (progn (rectangle-mark-mode 1)
-           (rectangle-next-line 2)
-           (rectangle-forward-char 8)
-           (rectangle-exchange-point-and-mark))
+    (rectangle-mark-mode 1)
+    (rectangle-next-line selected-window-contrast-region-lines) ; 2
+    (rectangle-forward-char selected-window-contrast-region-width) ; 8
+    (rectangle-exchange-point-and-mark)
     ;; Start timer to deactivate mark and rectangle mode.
     (run-with-timer selected-window-contrast-region-timeout
                     nil (lambda (buf)
@@ -268,7 +271,9 @@ Use `rectangle-mark-mode'.  Deactivate rectangle after 1 second or less."
                             (when (region-active-p)
                               ;; (exchange-point-and-mark)
                               (deactivate-mark))))
-                    (current-buffer))))
+                    (current-buffer)))
+  ;; save current window, because `previous-window' is not working.
+  (setq selected-window-contrast-prev-window (selected-window)))
 
 (defun selected-window-contrast-highlight-selected-window-with-timeout ()
   "Highlight not selected windows with a different background color.
@@ -283,8 +288,7 @@ $ emacsclient -c ~/file"
         (sw (selected-window)))
     (when (/= (aref cbn 0) ?\s) ; ignore system buffers
       ;; - not selected:
-      (when (or (eq selected-window-contrast-mode 1)
-                (eq selected-window-contrast-mode 3))
+      (when selected-window-contrast-flag
         (walk-windows (lambda (w)
                         (unless (or (eq sw w)
                                     (eq cbn (buffer-name (window-buffer w))))
@@ -293,16 +297,18 @@ $ emacsclient -c ~/file"
                             (selected-window-contrast-change-window
                              selected-window-contrast-bg-others
                              selected-window-contrast-text-others))))
-                      -1)) ; -1 means to not include minimuber
+                      -1)) ; -1 means to not include minibuffer
 
       ;; - selected:
-      (when (or (eq selected-window-contrast-mode 1)
-                (eq selected-window-contrast-mode 3))
-        (selected-window-contrast-change-window selected-window-contrast-bg-selected selected-window-contrast-text-selected))
-      (when (or (eq selected-window-contrast-mode 2)
-                (eq selected-window-contrast-mode 3))
-        (add-hook 'window-selection-change-functions #'selected-window-contrast-mark-small-rectangle-temporary nil t))
-      )))
+      (when selected-window-contrast-flag
+        (selected-window-contrast-change-window selected-window-contrast-bg-selected
+                                                selected-window-contrast-text-selected))
+      (if selected-window-contrast-region-flag
+        (add-hook 'window-selection-change-functions
+                  #'selected-window-contrast-mark-small-rectangle-temporary nil t)
+        ;; else
+        (remove-hook 'window-selection-change-functions
+                     #'selected-window-contrast-mark-small-rectangle-temporary t)))))
 
 (provide 'selected-window-contrast)
 ;;; selected-window-contrast.el ends here
